/*
  MeCab -- Yet Another Part-of-Speech and Morphological Analyzer
 
  $Id: tokenizer.h,v 1.16 2004/08/06 18:05:14 taku-ku Exp $;

  Copyright (C) 2001-2004 Taku Kudo <taku-ku@is.aist-nara.ac.jp>
  This is free software with ABSOLUTELY NO WARRANTY.
  
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.
  
  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.
  
  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/  
#ifndef _MECAB_TOKENIZER_H
#define _MECAB_TOKENIZER_H

#include "mecab.h"
#include "freelist.h"
#include "dictionary.h"
#include "param.h"
#include "common.h"

namespace MeCab {

  class Tokenizer 
  {
  private:
    FreeList<Node> nodeFreeList;
    unsigned int id;

  protected:
     
    Token bosToken, bosToken2, eosToken, unkToken;
    Node  *bosNode, *bosNode2, *eosNode;
    char*  unkFeature;
    char*  bosFeature;
    char*  eosFeature;
    Dictionary dic;
    std::string _what;

    inline char *skipCharClass (const char *begin, const char *end, 
			 int char_class, int &fail, unsigned int &next) const
    {
      register char *p = const_cast<char *>(begin);
      while (p != end && (fail = getCharClass (p, end, next)) == char_class) p += next;
      return p;
    }

    inline char *skipCharClass (const char *begin, const char *end, int char_class) const
    {
      register unsigned int next;
      register char *p = const_cast<char *>(begin);
      while (p != end && getCharClass (p, end, next) == char_class) p += next;
      return p;
    }
     
    virtual int getCharClass (const char *, const char *, unsigned int &) const = 0;

  public:

    // non vertual;
    Node *getNewNode ()
    { 
      Node *node = nodeFreeList.alloc ();
      node->id = id++;
      return node;
    }

    Node *getBOSNode ();
    Node *getEOSNode ();

    // pure-virtual functions
    virtual void preprocess  (const char *, const char *) = 0;
    virtual void postprocess (const char *, const char *) = 0;
    virtual Node *lookup     (const char *, const char *) = 0;

    virtual bool open (Param &);
    virtual bool close ();
    virtual void clear ();

    Tokenizer (Param &);
    Tokenizer ();
    virtual ~Tokenizer () { this->close (); };
  };
}
#endif
